/*
 * APPR - Automated Profiling and Performance Regression - is an
 * extensible performance profiling framework for OpenOffice.org.
 * 
 * Copyright (C) 2005 Intel Corporation
 * 
 * GNU Lesser General Public License Version 2.1
 * ======================================================================
 * 
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the
 * License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
 * USA
 * 
 * ======================================================================
 * 
 * Internal data structure definition for APPR hook
 *
 */

#ifndef __APPR_HOOK_INCLUDE__
#define __APPR_HOOK_INCLUDE__

#ifdef __standalone
#include "appr_hook_interface.h"
#else
#include <rtl/appr_hook_interface.h>
#endif // __standalone

#ifndef TRUE
#define TRUE 1
#endif // TRUE

#ifndef FALSE
#define FALSE 0
#endif // FALSE

#ifndef INVALID
#define INVALID -1
#endif // INVALID

#ifndef REGEX_MATCH
#define REGEX_MATCH ""
#endif // REGEX_MATCH

#ifndef REGEX_NONMATCH
#define REGEX_NONMATCH "1234567890098765432112345678900987654321"
#endif // REGEX_NONMATCH

#define APPR_HOOK_DAEMON_PORT 2777
#define APPR_HOOK_DAEMON_ID "/tmp/APPR_HOOK_DAEMON"
#define APPR_HOOK_ENVVAR_DAEMON_ID "APPR_HOOK_DAEMON_FOOTPRINT"

#define APPR_HOOK_ENVVAR_PROG "APPR_HOOK_PROG"

#define APPR_HOOK_ENVVAR_REGEX "APPR_HOOK_REGEX"
#define APPR_HOOK_ENVVAR_PAUSE_REGEX "APPR_HOOK_PAUSE_REGEX"
#define APPR_HOOK_ENVVAR_RESUME_REGEX "APPR_HOOK_RESUME_REGEX"
#define APPR_HOOK_ENVVAR_STARTPAUSED "APPR_HOOK_START_PAUSED"

#define APPR_HOOK_ENVVAR_OOO_RTL "APPR_HOOK_ENABLE_OOO_RTL"

#define APPR_HOOK_ENVVAR_VTUNE "APPR_HOOK_ENABLE_VTUNE"
#define APPR_HOOK_ENVVAR_VTUNE_STARTPAUSED APPR_HOOK_ENVVAR_STARTPAUSED

#define APPR_HOOK_ENVVAR_STRACE "APPR_HOOK_ENABLE_STRACE"
#define APPR_HOOK_ENVVAR_CALLGRIND "APPR_HOOK_ENABLE_CALLGRIND"

#define APPR_HOOK_COMMAND_MAXARG 10
#define APPR_HOOK_COMMAND_MAXLEN 256

#define APPR_HOOK_BUFFER_SIZE (APPR_HOOK_COMMAND_MAXLEN << 1)

#define LIB_VTUNEAPI "libVtuneApi.so"
#define FNT_VTUNEPAUSE "VTPause"
#define FNT_VTUNERESUME "VTResume"

enum {
	APPR_HOOK_TOKEN = 0xf0,
	APPR_HOOK_DESCRIPTION,
	APPR_HOOK_REQUEST,
	APPR_HOOK_REPLY,
	APPR_HOOK_CLOSE,
	APPR_HOOK_COMMAND_NUMBERS
};

struct apprbuf {
	unsigned char res[APPR_HOOK_BUFFER_SIZE];

	int start;
	int end;
};

struct apprcmd {
	unsigned char code;
	unsigned char size;
	unsigned char *argv;
	unsigned char *desc;
	unsigned char res[APPR_HOOK_COMMAND_MAXLEN];

	int argc;
	int failure;
	int expect_length;
	int command_length;
};

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

	int appr_hook_recv_command(int comm, struct apprcmd *command, struct apprbuf *buffer);
	int appr_hook_send_command(int comm, struct apprcmd *command, unsigned char code, char *desc, ...);

#ifdef __cplusplus
}
#endif // __cplusplus

#endif // __APPR_HOOK_INCLUDE__
