#!/bin/sh
#
# APPR - Automated Profiling and Performance Regression - is an
# extensible performance profiling framework for OpenOffice.org.
# 
# Copyright (C) 2005 Intel Corporation
# 
# GNU Lesser General Public License Version 2.1
# ======================================================================
# 
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
# USA
# 
# ======================================================================

BASEDIR=${0%/*}

if [ ${#1} -ne 0 ] && [ -d $1 ]; then
	TARGETDIR=$1
else
	echo "Usage:"
	echo -e "\t${0##*/} <OODIR>\tOODIR should be the base directory for OpenOffice.org build,"
	echo -e "\t\t\t\twhich has a child directory for sal module"
	echo
	echo -e "\tSample: ./apply.sh ooo-build-1.9.124/build/src680-m124"
	exit -1
fi

NEWLIST="sal/inc/rtl/appr_hook_interface.h
	sal/rtl/source/appr_hook.h
	sal/rtl/source/appr_hook.c
	sal/rtl/source/appr_hook_agent.c
	"

# patch the original files in OO source
DIFFLIST=$(cd $BASEDIR; find -type f | grep diff$)
for difffile in $DIFFLIST; do
	origfile=${difffile%.diff}
	backfile="$origfile-"
	if [ ! -f $TARGETDIR/$backfile ]; then
		$EXEC cp -p $TARGETDIR/$origfile $TARGETDIR/$backfile
	fi
	$EXEC patch $TARGETDIR/$origfile < $BASEDIR/$difffile
	if [ $? -eq 0 ]; then
		echo -e "\t[Succeed]"
	else
		echo -e "\t[Fail]"
	fi
done

# dymanically create symobl links
REALFILEDIR="../"
for newfile in $NEWLIST; do
	newfilename=${newfile##*/}
	srcfile=$BASEDIR/$REALFILEDIR/$newfilename
	dstfile=$TARGETDIR/$newfile
	echo "Copying file $dstfile" 
	$EXEC cp $srcfile $dstfile
	if [ $? -eq 0 ]; then
		echo -e "\t[Succeed]"
	else
		echo -e "\t[Fail]"
	fi
done

