/*
 * Process and system monitoring tool.
 *
 * (C) Eugeni Dodonov <eugeni.dodonov@mstech.com.br>, 2004-2005.
 *
 *      Max Alt <maxim.alt@intel.com>, 2004-2005
 * 
 * 	Xiaoyang Yu <xiaoyang.yu@intel.com>, 2005
 *
 */

#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>

#define KERNEL_BLOCK_SIZE 512 

int childended = 0;
int should_pause = 0;

void quit(int nothing)
{
	childended = 1;
}

void pause_it(int nothing)
{      
	should_pause=1;
}

void resume(int nothing)
{      
	should_pause=0;
}

int main(int argc, char **argv, char **env)
{
	FILE *file;
	unsigned long timer;
	unsigned long itemp, litemp, olduser, oldsystem, oldidle, oldnice, oldtotal, oldtx, oldrx, olduserproc, oldsystemproc;
	unsigned long disk_read, disk_write, disk_in = 0, disk_out = 0, disk_read_init, disk_write_init;
	unsigned long old_disk_read, old_disk_write, disk_tx_read, disk_tx_write;
	float maxuload, maxsload, maxpuload, maxsuload, usertime, systemtime;
	int pid;
	int do_io = 0;
	unsigned long maxrx, maxtx;
	float time = 0;
	char line[255], *interface, *disk, temp[10], ctemp, pname[255];

	/* moved from inside while loop */
	unsigned long user, sys, idle, nice, total, in, out; 
	unsigned long rx, tx, userproc, systemproc, vmem, maxvmem;
	long ltemp, rmem, maxrmem;
	double uload, sload, procuload, procsload;

	/* prefer using epoch time */
	struct timeval epoch_time;

	maxrmem= 0;
	maxvmem= 0;

	signal(10, pause_it);
	signal(12, resume);

	maxuload = maxsload = maxpuload = maxsuload = 0;
	maxrx = maxtx = 0;
	//maxrmem = maxvmem = 0;
	if(argc == 2){
	       if(strcmp(argv[1], "stop")==0){
			system("killall monitor");
			exit(0);
	       }
	       if(strcmp(argv[1], "pause")==0){
			system("killall -10 monitor");
			exit(0);
	       }
	       if(strcmp(argv[1], "resume")==0){
			system("killall -12 monitor");
			exit(0);
	       }
	}
	else if (argc < 5)
	{
		fprintf(stderr, "Usage: %s <delay in usecs> <net interface> <disk> <pid>|<command>\nOr: %s stop|pause|resume\n", 
			argv[0], argv[0]);
		exit(1);
	}
	timer = atoi(argv[1]);
	interface = argv[2];
	disk = argv[3];
	pid = atoi(argv[4]);

	if (pid <= 0)
	{
		/* Program */
		char *command;
		char **params;
		int i;
		command = (char *)malloc(strlen(argv[4]) * sizeof(char));
		params = (char **)malloc((argc - 4) * sizeof(char*));
		strncpy(command, argv[4], strlen(argv[4]));

		for (i=4; i < argc; i++)
		{
			params[i - 4] = (char *)malloc(strlen(argv[i]));
			strncpy(params[i-4], argv[i], strlen(argv[i]));
		}
		
		pid = fork();
		if (pid < 0)
		{
			perror("fork");
			exit(1);
		}
		if (pid == 0)
		{
			execve(command, params, env);
			perror("execve");
			exit(1);
		}
	}

	olduser = oldsystem = oldidle = oldnice = oldtotal = 0;

	/* CPU */
	file = fopen("/proc/stat", "r");
	fgets(line, 255, file);
	fclose(file);
	sscanf(line, "%s %lu %lu %lu %lu", temp, &olduser, &oldnice, &oldsystem,
			&oldidle);
	oldtotal = olduser + oldsystem + oldidle + oldnice;

	/* DISK */
	file = fopen("/proc/diskstats", "r");
	if (!file)
	{
		fprintf(stderr, "Error: Disk usage statistics supported for 2.6 kernels only for now.\n");
	}
	else
	{
		while (!feof(file))
		{
			fgets(line, 255, file);
			sscanf(line, "%lu %lu %s %lu %lu %lu %lu %lu %lu %lu",
					&itemp, &itemp, temp, &litemp, &litemp, &old_disk_read,
					&litemp, &litemp, &litemp, &old_disk_write);
			if (!strncmp(temp, disk, strlen(disk)) &&
					(strlen(temp) == strlen(disk)))
			{
				disk_read_init = old_disk_read;
				disk_write_init = old_disk_write;
				do_io = 1;
				break;
			}
		}
		fclose(file);
	}
	/* PROC */
	sprintf(line, "/proc/%d/stat", pid);
	file = fopen(line, "r");
	if (file == NULL)
	{
		fprintf(stderr, "Error: process (pid %d) not found!\n", pid);
		exit(1);
	}
	fgets(line, 255, file);
	fclose(file);
	// %d %s %c %d %d %d %d %d %lu %lu %lu %lu %lu | %lu %lu
	sscanf(line, "%lu %s %c %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu",
			&itemp, temp, &ctemp, &itemp, &itemp, &itemp, &itemp, &itemp,
			&litemp, &litemp, &litemp, &litemp, &litemp, &olduserproc, &oldsystemproc);

	/* Network */
	file = fopen("/proc/net/dev", "r");
	fgets(line, 255, file);
	fgets(line, 255, file);
	while (!feof(file))
	{
		int i;
		fgets(line, 255, file);
		strncpy(temp, line, 6);
		sscanf(line + 7, "%lu %lu %lu %lu %lu %lu %lu %lu %lu", &oldrx,
				&itemp, &itemp, &itemp, &itemp, &itemp,
				&itemp, &itemp, &oldtx);
		for (i=0; i < 6; i++)
			if (temp[i] != ' ') break;
		if (!strncmp(temp + i, interface, strlen(interface)))
			break;
		else
			oldrx = oldtx = 0;
	}
	fclose(file);
	usertime = systemtime = 0;

	while(1){
		if(!should_pause)
		{
			usleep(timer);

			/* CPU */
			file = fopen("/proc/stat", "r");
			fgets(line, 255, file);
			fclose(file);
			sscanf(line, "%s %lu %lu %lu %lu", temp, &user, &nice, &sys,
					&idle);
			total = user + sys + idle + nice;
			uload = (double) ((user - olduser) * 100) / (total - oldtotal);
			sload = (double) ((sys - oldsystem) * 100) / (total - oldtotal);
			time += ((double)timer / 1000000);
			gettimeofday(&epoch_time, 0);		       

			/* DISK */
			/* XXX: We could not have this data! Check! */
			if (do_io)
			{
				file = fopen("/proc/diskstats", "r");
				if (!file)
				{
					fprintf(stderr, "Error: Disk usage statistics supported for 2.6 kernels only for now.\n");
				}
				else
				{
					while (!feof(file))
					{
						fgets(line, 255, file);
						sscanf(line, "%lu %lu %s %lu %lu %lu %lu %lu %lu %lu",
								&itemp, &itemp, temp, &litemp, &litemp, &disk_read,
								&litemp, &litemp, &litemp, &disk_write);
						if (!strncmp(temp, disk, strlen(disk)) &&
								(strlen(temp) == strlen(disk)))
						{
							disk_tx_read = disk_read - old_disk_read;
							disk_tx_write = disk_write - old_disk_write;
							disk_in = disk_read - disk_read_init;
							disk_out = disk_write - disk_write_init;
							old_disk_read = disk_read;
							old_disk_write = disk_write;
							break;
						}
					}
					fclose(file);
				}
			}

			/* PROC */
			sprintf(line, "/proc/%d/stat", pid);
			file = fopen(line, "r");
			if (file == NULL)
			{
				fprintf(stderr, "Error: process (pid %d) not found!\n", pid);
				exit(1);
			}
			fgets(line, 255, file);
			fclose(file);
			// %d %s %c %d %d %d %d %d %lu %lu %lu %lu %lu | %lu %lu
			sscanf(line, "%lu %s %c %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "\
					"%ld %ld %ld %ld %ld %ld %lu %lu %ld",
					&itemp, pname, &ctemp, &itemp, &itemp, &itemp, &itemp, &itemp,
					&litemp, &litemp, &litemp, &litemp, &litemp, &userproc, &systemproc,
					&ltemp, &ltemp, &ltemp, &ltemp, &ltemp, &ltemp, &litemp, &vmem, &rmem
					);
			usertime = (double)userproc / 100;
			systemtime = (double)systemproc / 100;

			/* Network */
			file = fopen("/proc/net/dev", "r");
			fgets(line, 255, file);
			fgets(line, 255, file);
			while (!feof(file))
			{
				int i=0;
				fgets(line, 255, file);
				strncpy(temp, line, 6);
				sscanf(line + 7, "%lu %lu %lu %lu %lu %lu %lu %lu %lu", &rx,
						&itemp, &itemp, &itemp, &itemp, &itemp,
						&itemp, &itemp, &tx);
				for (i=0; i < 6; i++)
					if (temp[i] != ' ') break;
				if (!strncmp(temp + i, interface, strlen(interface)))
				{
					break;
				}
				else
					rx = tx = 0;
			}
			fclose(file);

			in = rx - oldrx;
			out = tx - oldtx;
			oldrx = rx;
			oldtx = tx;

			if ((userproc == olduserproc) || (user == olduser))
				procuload = 0.0;
			else
			{
				procuload = ((double)(userproc - olduserproc) / (double)(user - olduser) * 100);
				procuload *= uload / 100;
			}

			if ((systemproc == oldsystemproc) || (sys == oldsystem))
				procsload = 0.0;
			else
			{
				procsload = ((double)(systemproc - oldsystemproc) / (double)(sys - oldsystem) * 100);
				procsload *= sload / 100;
			}
			olduserproc = userproc;
			oldsystemproc = systemproc;

			if (procuload > 100) procuload = 100;
			if (procsload > 100) procsload = 100;
			if (procuload > maxpuload) maxpuload = procuload;
			if (procsload > maxsuload) maxsuload = procsload;
			if (uload > maxuload) maxuload = uload;
			if (sload > maxsload) maxsload = sload;
			if (in > maxrx) maxrx = in;
			if (out > maxtx) maxtx = out;
			if (vmem > maxvmem) maxvmem = vmem;
			if (rmem > maxrmem) maxrmem = rmem;

#ifndef VERBOSE
				printf("<Stats pname=\"%s\" pid=\"%d\" time=\"%f\" usertime=\"%.1f\" "
				       "systemtime=\"%.1f\" procuserload=\"%.1f\" maxprocuserload=\"%.1f\" "
				       "procsysload=\"%.1f\" masprocsysload=\"%.1f\" userload=\"%.1f\" "
				       "maxuserload=\"%.1f\" systemload=\"%.1f\" maxsystemload=\"%.1f\" "
				       "vmem=\"%lu\" maxvmem=\"%lu\" rmem=\"%lu\" maxrmem=\"%lu\" bytesin=\"%lu\" "
				       "maxbytesin=\"%lu\" "
				       "bytesout=\"%lu\" maxbytesout=\"%lu\" "
				       "disk_throughput_read=\"%lu\" disk_throughput_write=\"%lu\" "
				       "disk_bytes_read=\"%lu\" disk_bytes_write=\"%lu\" "
				       "/>\n",
				       pname, pid,
				       epoch_time.tv_sec + epoch_time.tv_usec / 1e6, usertime, systemtime, procuload, maxpuload, 
				       procsload, maxsuload, uload, maxuload,
				       sload, maxsload, vmem, maxvmem, rmem, maxrmem, in, maxrx, out, maxtx,
				       disk_tx_read * KERNEL_BLOCK_SIZE, disk_tx_write * KERNEL_BLOCK_SIZE, disk_in * KERNEL_BLOCK_SIZE, 
				       disk_out * KERNEL_BLOCK_SIZE);
#endif
#ifdef VERBOSE
				printf("Monitoring %s (PID %d)\nMonitoring time:\t\t%5.1fs\nProcess USER time:\t\t%5.1fs\n"\
					"Process SYSTEM time:\t\t%5.1fs\n"\
					"Current process USER usage:\t%5.1f%% (%5.1f%%)\n"\
					"Current process SYSTEM usage:\t%5.1f%% (%5.1f%%)\n"\
					"System USER load:\t\t%5.1f%% (%5.1f%%)\n"\
					"System SYSTEM load:\t\t%5.1f%% (%5.1f%%)\n"\
					"Memory VIRTUAL size:\t\t%lu (%lu)\n"\
					"Memory RESIDENT size:\t\t%d\n"\
					"Network IN:\t\t%10lu (%10lu)\n"\
					"Network OUT:\t\t%10lu (%10lu)\n\n",
					pname, pid,
					time, usertime, systemtime, procuload, maxpuload, procsload, maxsuload, uload, maxuload, sload, maxsload,
					vmem, maxvmem, rmem, in, maxrx, out, maxtx);
#endif

			fflush(stdout);
			olduser = user;
			oldsystem = sys;
			oldidle = idle;
			oldnice = nice;
			oldtotal = total;
		}
		else
			usleep(1);
	}
	return 0;
}
