/*
 * APPR - Automated Profiling and Performance Regression - is an
 * extensible performance profiling framework for OpenOffice.org.
 * 
 * Copyright (C) 2005 Intel Corporation
 * 
 * GNU Lesser General Public License Version 2.1
 * ======================================================================
 * 
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the
 * License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
 * USA
 * 
 * ======================================================================
 * 
 * Utility to pause and then load an application after a resume singal received
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <regex.h>
#include <unistd.h>
#include <signal.h>

#define APPR_HOOK_ATTACH "APPR_HOOK_ATTACH"
#define APPR_HOOK_TARGET_PID "APPR_HOOK_TARGET_PID"
#define SIGNAL_RESUME SIGCONT
#define MAXSLEEP 10

typedef void (*sighandler_t)(int);

void sigHandle_resume(int signum)
{
  fprintf(stderr, "Signal RESUME caught.\n");
}

int main(int argc, char* argv[])
{
	int value, i;
	char *penv;
	char buf[1000];
	regex_t pattern;
	regmatch_t match;
	sigset_t sig_mask, osig_mask;

	if (argc >= 2) {

		/* register resume signal */
		signal(SIGNAL_RESUME, (sighandler_t)sigHandle_resume);

		/* sed environment variable to provide PID for current process */
		sprintf(buf, "%d", getpid());
		setenv(APPR_HOOK_TARGET_PID, buf, 1);

		/* set signal mask to only enable resume signal */
		sigfillset(&sig_mask);
		sigdelset(&sig_mask, SIGNAL_RESUME);
		sigprocmask(SIG_SETMASK, &sig_mask, &osig_mask);

		// pause();
		sleep(MAXSLEEP);

		/* recover signal mask */
		sigprocmask(SIG_SETMASK, &osig_mask, NULL);

		/* invoke the "ATTACH" hook */
		penv = getenv(APPR_HOOK_ATTACH);
		if (penv != NULL) {
			system(penv);
		} else {
			fprintf(stderr, "APPR ATTACH hook not defined.\n");
		}

		/* recover the command line options */
		regcomp(&pattern, "#SEMICOLON#", REG_EXTENDED);
		for (i = 0; i < argc; i++) {
			value = regexec(&pattern, argv[i], 1, &match, 0);
			while ((value == 0) && (match.rm_so >= 0) && (match.rm_eo >= 0)) {
				strcpy(&argv[i][match.rm_so+1], &argv[i][match.rm_eo]);
				argv[i][match.rm_so] = ',';
				value = regexec(&pattern, argv[i], 1, &match, 0);
			}
			// fprintf(stderr, "%s ", argv[i]);
		}
		// fprintf(stderr, "\n");

		return execv(argv[1], &(argv[1]));
	} else {
		fprintf(stderr, "Usage:\n");
		fprintf(stderr, "\t%s <application> [options]\n", argv[0]);
		return -1;
	}
}

